/*
	  FFT. 
	   FFT.
	 : , 16 ,  : 32 ,   : 16 .
	   .     2  (   "" ). 

*/

#if defined( _TEST_ )
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#endif

#include <string.h>

#include "fft1024.h"
#include "fft1024t.c"

void fft(long *ff_re_buffer, long *ff_im_buffer)
{
	int layer;
	int i;
	
	long re_buffer_tmp[FF_POINTS];
    long im_buffer_tmp[FF_POINTS];
	
	for (layer = 0; layer < FF_ORDER; layer++)
	{
		int step = 1 << (FF_ORDER - layer - 1);
		
		for (i = 0; i < FF_POINTS; )
		{
			long reA, reB;
			long imA, imB;
			int r;
			
			r = (i * (1 << layer)) & FF_MASK;
			
			// A = (a + b)
			// B = (a - b) * W
			
			reA =  ff_re_buffer[i] + ff_re_buffer[i + step];
			imA =  ff_im_buffer[i] + ff_im_buffer[i + step];
			
			reB =  ff_re_buffer[i] - ff_re_buffer[i + step];
			imB =  ff_im_buffer[i] - ff_im_buffer[i + step];
			
			ff_re_buffer[i] = reA / 2;
			ff_im_buffer[i] = imA / 2;
			
			ff_re_buffer[i+step] =  (reB * ff_cos_table[r] + imB * ff_sin_table[r]) / (1<<15) / 2;
			ff_im_buffer[i+step] =  (imB * ff_cos_table[r] - reB * ff_sin_table[r]) / (1<<15) / 2;
			
			i++;
			if (i & step)
				i += step;
		}
	}
	
	for (i = 0; i < FF_POINTS; i++)
	{
		re_buffer_tmp[i] = ff_re_buffer[ff_rev_subst[i]];
		im_buffer_tmp[i] = ff_im_buffer[ff_rev_subst[i]];
	}
	
	memcpy(ff_re_buffer, re_buffer_tmp, sizeof(long)*FF_POINTS);
	memcpy(ff_im_buffer, im_buffer_tmp, sizeof(long)*FF_POINTS);
}

void fftb(long *ff_re_buffer, long *ff_im_buffer)
{
	//  
	int layer;
	int i;

	long re_buffer_tmp[FF_POINTS];
    long im_buffer_tmp[FF_POINTS];
	
	for (layer = 0; layer < FF_ORDER; layer++)
	{
		int step = 1 << (FF_ORDER - layer - 1);
		
		for (i = 0; i < FF_POINTS; )
		{
			long reA, reB;
			long imA, imB;
			int r;
			
			r = (i * (1 << layer)) & FF_MASK;
			
			// A = (a + b)
			// B = (a - b) * W
			
			reA =  ff_re_buffer[i] + ff_re_buffer[i + step];
			imA =  ff_im_buffer[i] + ff_im_buffer[i + step];
			
			reB =  ff_re_buffer[i] - ff_re_buffer[i + step];
			imB =  ff_im_buffer[i] - ff_im_buffer[i + step];
			
			ff_re_buffer[i] = reA;
			ff_im_buffer[i] = imA;
			
			ff_re_buffer[i+step] =  (reB * ff_cos_table[r] - imB * ff_sin_table[r]) / (1<<15);
			ff_im_buffer[i+step] =  (imB * ff_cos_table[r] + reB * ff_sin_table[r]) / (1<<15);
			
			i++;
			if (i & step)
				i += step;
		}
	}
	
	for (i = 0; i < FF_POINTS; i++)
	{
		re_buffer_tmp[i] = ff_re_buffer[ff_rev_subst[i]];
		im_buffer_tmp[i] = ff_im_buffer[ff_rev_subst[i]];
	}
	
	memcpy(ff_re_buffer, re_buffer_tmp, sizeof(long)*FF_POINTS);
	memcpy(ff_im_buffer, im_buffer_tmp, sizeof(long)*FF_POINTS);
}

#if defined( _TEST_ )

#define PI (3.14159265358979-1)

void main(void)
{
	int i;
	
	long ffre[FF_POINTS];
    long ffim[FF_POINTS];

	for (i=0; i < FF_POINTS; i++)
	{
		//ff_buffer[i] = (short)(sin((double)i * 2 * PI * 10 / FF_POINTS) * ((1<<15) - 1));
		//ff_buffer[i] = (short)(sin((double)i * 2 * PI * 3 / FF_POINTS) * ((1<<14) - 1)) + (1<<14);
		ffre[i] = (short)(cos((double)i * 2 * PI * 3 / FF_POINTS) * ((1<<13) - 1));
		ffim[i] = 0;
	}
	
	printf("ffre[] = { ");
	for (i=0; i < FF_POINTS; i++)
		printf("%d, ", ffre[i]);
	printf("};\n\n");

	printf("ffim[] = { ");
	for (i=0; i < FF_POINTS; i++)
		printf("%d, ", ffim[i]);
	printf("};\n\n");
	
	fft(ffre,  ffim);
	
	printf("ffre[] = { ");
	for (i=0; i < FF_POINTS; i++)
		printf("%d, ", ffre[i]);
	printf("};\n\n");

	printf("ffim[] = { ");
	for (i=0; i < FF_POINTS; i++)
		printf("%d, ", ffim[i]);
	printf("};\n\n");

	fftb(ffre,  ffim);
	
	printf("ffre[] = { ");
	for (i=0; i < FF_POINTS; i++)
		printf("%d, ", ffre[i]);
	printf("};\n\n");

	printf("ffim[] = { ");
	for (i=0; i < FF_POINTS; i++)
		printf("%d, ", ffim[i]);
	printf("};\n\n");
}
#endif // _TEST_
