#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <math.h>

#include "bmp.h"
#include "fft1024.h"
#include "wav.h"

#define GFG_VERSION		0x000001
#define MAX_COLOR		0x1000
#define _COLORCONST		4.0


void help(void)
{
	printf("GipFractal Generator v. %02x.%02x.%02x\r\n", 
		(GFG_VERSION&0xFF0000)>>16, 
		(GFG_VERSION&0xFF00)>>8,
		(GFG_VERSION&0xFF));
	printf("\n");
	printf("\n");
	printf("\n");
	printf("\n");
	exit(0);
}

RGBQUAD palette[256];

#define SIZE_X 1024


int main(int argc, char **argv)
{
	bmp_struct_t bmp;
	wav_struct_t wav;
	wav_struct_t wav2;
	wav_info_struct_t wavinfo;
	
	int i, j, y;
	
	uint32_t sample[4];

	if (wav_load(&wav, "sample.wav"))
	{
		printf("error 1\r\n");
		return 0;
	}
	
	if (wav_info(&wav, &wavinfo))
	{
		printf("error 2\r\n");
		return 0;
	}

	if (wav_create(&wav2, &wavinfo))
	{
		printf("error 3\r\n");
		return 0;
	}
	
	printf("dwSamplesPerSec = %d\r\n", wavinfo.dwSamplesPerSec);
	printf("wBitsPerSample  = %d\r\n", wavinfo.wBitsPerSample);
	printf("wChannels       = %d\r\n", wavinfo.wChannels);
	printf("Samples N       = %d\r\n", wavinfo.dwDataLen);

	bmp_create(&bmp, wavinfo.dwDataLen / 1024 * 2, SIZE_X/2, 8);
	bmp_fillrect(&bmp, 0, 0, 255, SIZE_X/2, wavinfo.dwDataLen / 1024 * 2);
	
	for (i = 0; i < 256; i++)
	{
		palette[i].rgbRed = i;
		palette[i].rgbGreen = i;
		palette[i].rgbBlue = i;
	}
	
	bmp_setpalette8(&bmp, palette);
	
	y = 0;
	for (i = 0; i < wavinfo.dwDataLen; i++)
	{
		long re_buffer[FF_POINTS];
		long im_buffer[FF_POINTS];
		
		for (j = 0; j < SIZE_X; j++)
		{
			wav_get_sample(&wav, i+j, sample);
			
			//re_buffer[j] = sample[0] << 8;
			re_buffer[j] = sample[0];
			im_buffer[j] = 0;
			
/*
			if (i == 0)
			{
				printf("[%02x|%02x]", re_buffer[j], im_buffer[j]);
				if ((j+1) % 16 == 0)
					printf("\r\n");
			}
*/
		}
		
/*
		if (i == 0)
		{
			printf("\r\n");
		}
*/
		fft(re_buffer, im_buffer);
		
		for (j = 0; j < SIZE_X/2; j++)
		{
			short tmp = (short)(sqrt((long)re_buffer[j] * re_buffer[j] + (long)im_buffer[j] * im_buffer[j]));
			
			if ((tmp >> 2) > 255)
				bmp_putpixel(&bmp, j, y, 255);
			else
				bmp_putpixel(&bmp, j, y, tmp >> 2);
			
/*			if (i == 0)
			{
				printf("[%02x|%02x]", re_buffer[j], im_buffer[j]);
				if ((j+1) % 16 == 0)
					printf("\r\n");
			}
*/
		}
		
		if (i == 0)
		{
			printf("\r\n");
		}
		
		if ((i + SIZE_X) > wavinfo.dwDataLen)
			break;
		
		fftb(re_buffer, im_buffer);
		
		for (j = 0; j < SIZE_X; j++)
		{
			sample[0] = re_buffer[j];
			sample[1] = re_buffer[j];
			sample[2] = re_buffer[j];
			sample[3] = re_buffer[j];
			
			//sample[0] = re_buffer[j] >> 8;
			//sample[1] = re_buffer[j] >> 8;
			//sample[2] = re_buffer[j] >> 8;
			//sample[3] = re_buffer[j] >> 8;
			
			if (wav_set_sample(&wav2, i+j, sample))
			{
				printf("error 4\r\n");
				return 0;
			}
			
/*
			if (i == 0)
			{
				printf("[%02x|%02x]", re_buffer[j], im_buffer[j]);
				if ((j+1) % 16 == 0)
					printf("\r\n");
			}
*/
		}
		
		i += j / 2;
		y++;
	}

	bmp_save(&bmp, "test.bmp");
	bmp_close(&bmp);
	printf("ok\r\n");
	
	wav_save(&wav2, "test.wav");
	wav_close(&wav2);
	wav_close(&wav);

	return 0;

/*
	bmp_create(&bmp, SIZE_Y, SIZE_X, 8);
	
	for (i = 0; i < 256; i++)
	{
		palette[i].rgbRed = i;
		palette[i].rgbGreen = i;
		palette[i].rgbBlue = i;
	}
	
	bmp_setpalette8(&bmp, palette);
	
	bmp_fillrect(&bmp, 0, 0, 255, SIZE_X, SIZE_Y);
	
	for (i = 0; i < SIZE_X; i++)
	{
		double x_var;
		double k_var = (double)i * 3 / SIZE_X;
		
		//x_var = 0.54321;
		x_var = 0.1;
		
		if (i == SIZE_X / 3 || i == SIZE_X * 2 / 3)
			for (j = 0; j < SIZE_Y; j++)
			{
				bmp_putpixel(&bmp, i, j, 200);
			}
		
		for (j = 0; j < 100; j++)
		{
			int y;
			
			x_var = k_var * x_var * x_var - 1;
			
			// x_var = [-1.0; 1.0];
			y = (x_var + 1) * SIZE_Y / 2;
		}

		for (      ; j < 1000; j++)
		{
			int y;
			
			x_var = k_var * x_var * x_var - 1;
			
			// x_var = [-1.0; 1.0];
			y = (x_var + 1) * SIZE_Y / 2;
			
			if (y >= 0 && y < SIZE_Y)
				bmp_putpixel(&bmp, i, y, 0);
		}
	}

	bmp_putstr(&bmp, 0, SIZE_Y-12-12, "k=", 0);
	bmp_putstr(&bmp, 0, SIZE_Y-12, "[0]", 0);
	bmp_putstr(&bmp, SIZE_X/3, SIZE_Y-12, "[1]", 0);
	bmp_putstr(&bmp, 2*SIZE_X/3, SIZE_Y-12, "[2]", 0);
	bmp_putstr(&bmp, 50, 200, "kx^2-1", 0);

	bmp_save(&bmp, "test.bmp");
	bmp_close(&bmp);
	printf("ok\r\n");

	return 0;
*/
}
